import 'package:eyflutter_core/dio/beans/retrofit.dart';
import 'package:eyflutter_core/dio/dio_manager.dart';
import 'package:eyflutter_core/dio/enums/method.dart';
import 'package:eyflutter_core/ey_flutter.dart';
import 'package:eyflutter_core/kit/utils/dynamic_utils.dart';
import 'package:eyflutter_core/kit/utils/json_utils.dart';
import 'package:eyflutter_core/kit/utils/map/map_extension.dart';
import 'package:eyflutter_core/kit/utils/string/json_extension.dart';
import 'package:eyflutter_core/kit/utils/string/string_extension.dart';
import 'package:eyflutter_core/kit/utils/string/url_extension.dart';
import 'package:eyflutter_core/storage/mmkv_utils.dart';

class ConfigTable {
  factory ConfigTable() => _getInstance();

  static ConfigTable get instance => _getInstance();
  static ConfigTable? _instance;

  static ConfigTable _getInstance() {
    return _instance ??= new ConfigTable._internal();
  }

  ConfigTable._internal();

  Map<String, String> _tempMap = {};
  String _configCacheKey = "d954677b2383f5f9";

  /// 获取配置值
  /// [configKey] 配置表健
  Future<String> getValue({required String configKey}) {
    if (_tempMap.containsKey(configKey)) {
      var value = _tempMap.getValue<String>(configKey);
      return Future.value(value);
    }
    return _fromCacheTake(configKey);
  }

  /// 获取配置值
  /// [configKey] 配置表健
  Future<int> getDigitValue({required String configKey}) async {
    var value = await this.getValue(configKey: configKey);
    return Future.value(value.toInt);
  }

  /// 获取配置值
  /// [configKey] 配置表健
  Future<bool> getStateValue({required String configKey}) async {
    var value = await this.getValue(configKey: configKey);
    return Future.value(value.isTrue);
  }

  Future<String> _match(String configKey, String configData, bool isFromCache) {
    String matchValue = "";
    Map<String, dynamic> map = JsonUtils.fromJson(configData);
    map.forEach((key, value) {
      if (value is String) {
        if (key == configKey) {
          matchValue = value.trim();
        }
        _tempMap[key] = value;
      }
    });
    if (isFromCache && matchValue.isEmptyString) {
      return _fromNetTake(configKey);
    } else {
      return Future.value(matchValue);
    }
  }

  Future<String> _fromCacheTake(String configKey) async {
    var configData = await MmkvUtils.instance.getString(configKey);
    if (configData.isEmptyJson) {
      return _fromNetTake(configKey);
    }
    return _match(configKey, configData!, true);
  }

  Future<String> _fromNetTake(String configKey) async {
    var config = EyFlutter.instance.applicationConfig;
    var url = config?.configUrl() ?? "";
    if (!url.isUrl) {
      return Future.value("");
    }
    Retrofit retrofit = Retrofit(api: url, method: Method.GET);
    var response = await DioManager.instance.asyncRequestString(retrofit: retrofit);
    if (response.isEmptyString) {
      return Future.value("");
    }
    MmkvUtils.instance.putString(_configCacheKey, value: response);
    return _match(configKey, response, false);
  }
}
